#ifndef QT_FEATURES_Tools_src_global_qttools_config_p_h_H
#define QT_FEATURES_Tools_src_global_qttools_config_p_h_H

#define QT_FEATURE_assistant 1

#define QT_FEATURE_clang 1

#define QT_FEATURE_clang_rtti 1

#define QT_FEATURE_qdoc 1

#define QT_FEATURE_qdoc_coverage -1

#define QT_FEATURE_designer 1

#define QT_FEATURE_distancefieldgenerator 1

#define QT_FEATURE_kmap2qmap 1

#define QT_FEATURE_linguist 1

#define QT_FEATURE_pixeltool 1

#define QT_FEATURE_qdbus 1

#define QT_FEATURE_qev 1

#define QT_FEATURE_qtattributionsscanner 1

#define QT_FEATURE_qtdiag 1

#define QT_FEATURE_qtplugininfo 1


#endif // QT_FEATURES_Tools_src_global_qttools_config_p_h_H
